Configuration management for MCP OAuth server
This module provides environment variable loading functions following the principle of single responsibility and making configuration explicit and testable.
Enums
- Config
Error - Configuration errors
Functions
- get_
bind_ socket_ addr - Get server bind address as SocketAddr
- get_
cognito_ client_ id - AWS Cognito OAuth configuration values
- get_
cognito_ client_ secret - get_
cognito_ domain - get_
cognito_ oauth_ provider_ config - Cognito OAuth provider configuration
- get_
cognito_ region - get_
cognito_ scope - get_
cognito_ user_ pool_ id - get_
github_ client_ id - GitHub OAuth configuration values
- get_
github_ client_ secret - get_
github_ oauth_ provider_ config - GitHub OAuth provider configuration
- get_
github_ scope - get_
logging_ format - get_
logging_ level - Logging configuration values
- get_
oauth_ provider_ config - OAuth provider configuration values
- get_
server_ bind_ address - get_
server_ description - get_
server_ host - Server configuration values
- get_
server_ name - get_
server_ port - get_
server_ version